@@ -51,14 +51,23 @@ class CORE_EXPORT QgsRendererCategory
     * The \a label argument specifies the label used for this category in legends and the layer tree.
     *
     * The \a render argument indicates whether the category should initially be rendered and appear checked in the layer tree.
+    *
+    * The optional \a uuid argument manually set the UUID key identifier for the category (since QGIS 3.34).
     */
-    QgsRendererCategory( const QVariant &value, QgsSymbol *symbol SIP_TRANSFER, const QString &label, bool render = true );
+    QgsRendererCategory( const QVariant &value, QgsSymbol *symbol SIP_TRANSFER, const QString &label, bool render = true, const QString &uuid = QString() );
 
     /**
      * Copy constructor.
      */
     QgsRendererCategory( const QgsRendererCategory &cat );
     QgsRendererCategory &operator=( QgsRendererCategory cat );
+    ~QgsRendererCategory();
+
+    /**
+     * Returns the unique identifier for this category.
+     * \since QGIS 3.34
+     */
+    QString uuid() const;
 
     /**
      * Returns the value corresponding to this category.
@@ -110,14 +119,12 @@ class CORE_EXPORT QgsRendererCategory
     /**
      * Returns TRUE if the category is currently enabled and should be rendered.
      * \see setRenderState()
-     * \since QGIS 2.5
      */
     bool renderState() const;
 
     /**
      * Sets whether the category is currently enabled and should be rendered.
      * \see renderState()
-     * \since QGIS 2.5
      */
     void setRenderState( bool render );
 
@@ -131,13 +138,26 @@ class CORE_EXPORT QgsRendererCategory
     /**
      * Converts the category to a matching SLD rule, within the specified DOM document and \a element.
      */
-    void toSld( QDomDocument &doc, QDomElement &element, QgsStringMap props ) const;
+    void toSld( QDomDocument &doc, QDomElement &element, QVariantMap props ) const;
+
+#ifdef SIP_RUN
+    SIP_PYOBJECT __repr__();
+    % MethodCode
+    const QString str = !sipCpp->value().isValid()
+                        ? QStringLiteral( "<QgsRendererCategory>" )
+                        : sipCpp->label().isEmpty()
+                        ? QStringLiteral( "<QgsRendererCategory: %1>" ).arg( sipCpp->value().toString() )
+                        : QStringLiteral( "<QgsRendererCategory: %1 (%2)>" ).arg( sipCpp->value().toString(), sipCpp->label() );
+    sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+    % End
+#endif
 
   protected:
     QVariant mValue;
     std::unique_ptr<QgsSymbol> mSymbol;
     QString mLabel;
     bool mRender = true;
+    QString mUuid;
 
     void swap( QgsRendererCategory &other );
 };
@@ -171,7 +191,7 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
     bool filterNeedsGeometry() const override;
     QString dump() const override;
     QgsCategorizedSymbolRenderer *clone() const override SIP_FACTORY;
-    void toSld( QDomDocument &doc, QDomElement &element, const QgsStringMap &props = QgsStringMap() ) const override;
+    void toSld( QDomDocument &doc, QDomElement &element, const QVariantMap &props = QVariantMap() ) const override;
     QgsFeatureRenderer::Capabilities capabilities() override { return SymbolLevels | Filter; }
     QString filter( const QgsFields &fields = QgsFields() ) override;
     QgsSymbolList symbols( QgsRenderContext &context ) const override;
@@ -197,7 +217,6 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
 
     /**
      * Returns the index of the category with the specified label (or -1 if the label was not found, or is not unique).
-     * \since QGIS 2.5
      */
     int categoryIndexForLabel( const QString &val );
 
@@ -245,7 +264,6 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
      * \see updateCategorySymbol()
      * \see updateCategoryLabel()
      *
-     * \since QGIS 2.5
      */
     bool updateCategoryRenderState( int catIndex, bool render );
 
@@ -315,6 +333,7 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
     QDomElement save( QDomDocument &doc, const QgsReadWriteContext &context ) override;
     QgsLegendSymbolList legendSymbolItems() const override;
     QSet< QString > legendKeysForFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
+    QString legendKeyToExpression( const QString &key, QgsVectorLayer *layer, bool &ok ) const override;
 
     /**
      * Returns the renderer's source symbol, which is the base symbol used for the each categories' symbol before applying
@@ -368,7 +387,6 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
     /**
      * Update the color ramp used and all symbols colors.
       * \param ramp color ramp. Ownership is transferred to the renderer
-      * \since QGIS 2.5
       */
     void updateColorRamp( QgsColorRamp *ramp SIP_TRANSFER );
 
@@ -379,11 +397,13 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
     QString legendClassificationAttribute() const override { return classAttribute(); }
 
     /**
-     * creates a QgsCategorizedSymbolRenderer from an existing renderer.
-     * \returns a new renderer if the conversion was possible, otherwise 0.
-     * \since QGIS 2.5
+     * Creates a new QgsCategorizedSymbolRenderer from an existing \a renderer.
+     *
+     * Since QGIS 3.20, the optional \a layer parameter is required for conversions of some renderer types.
+     *
+     * \returns a new renderer if the conversion was possible, otherwise NULLPTR.
      */
-    static QgsCategorizedSymbolRenderer *convertFromRenderer( const QgsFeatureRenderer *renderer ) SIP_FACTORY;
+    static QgsCategorizedSymbolRenderer *convertFromRenderer( const QgsFeatureRenderer *renderer, QgsVectorLayer *layer = nullptr ) SIP_FACTORY;
 
     /**
      * Configures appearance of legend when renderer is configured to use data-defined size for marker symbols.
@@ -393,14 +413,12 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
      * When renderer does not use data-defined size or does not use marker symbols, these settings will be ignored.
      * Takes ownership of the passed settings objects. NULLPTR is a valid input that disables data-defined
      * size legend.
-     * \since QGIS 3.0
      */
     void setDataDefinedSizeLegend( QgsDataDefinedSizeLegend *settings SIP_TRANSFER );
 
     /**
      * Returns configuration of appearance of legend when using data-defined size for marker symbols.
      * Will return NULLPTR if the functionality is disabled.
-     * \since QGIS 3.0
      */
     QgsDataDefinedSizeLegend *dataDefinedSizeLegend() const;
 
@@ -421,7 +439,7 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
      *
      * \since QGIS 3.4
      */
-    int matchToSymbols( QgsStyle *style, QgsSymbol::SymbolType type,
+    int matchToSymbols( QgsStyle *style, Qgis::SymbolType type,
                         QVariantList &unmatchedCategories SIP_OUT, QStringList &unmatchedSymbols SIP_OUT, bool caseSensitive = true, bool useTolerantMatch = false );
 
 
@@ -437,6 +455,18 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
      */
     static QgsCategoryList createCategories( const QVariantList &values, const QgsSymbol *symbol, QgsVectorLayer *layer = nullptr, const QString &fieldName = QString() );
 
+    /**
+     *  Returns a localized representation of \a value with the given \a precision,
+     *  if precision is -1 then precision is guessed from the default QVariant::toString
+     *  output.
+     *
+     *  \note Precision is ignored for integers.
+     *
+     *  \since QGIS 3.22.1
+     */
+    static QString displayString( const QVariant &value, int precision = -1 );
+
+
   protected:
     QString mAttrName;
     QgsCategoryList mCategories;
@@ -494,6 +524,7 @@ class CORE_EXPORT QgsCategorizedSymbolRenderer : public QgsFeatureRenderer
 
     //! Returns list of legend symbol items from individual categories
     QgsLegendSymbolList baseLegendSymbolItems() const;
+
 };
 
 #endif // QGSCATEGORIZEDSYMBOLRENDERER_H
