@@ -9,21 +9,21 @@
 // or else upgrading old project file will not work
 // reliably.
 // TODO QGIS 4: remove in favor of _QGIS_VERSION
-#define VERSION "3.34.3-Prizren"
-#define _QGIS_VERSION "3.34.3-Prizren"
+#define VERSION "3.36.0-Maidenhead"
+#define _QGIS_VERSION "3.36.0-Maidenhead"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 3343
+//#define VERSION_INT 3360
 // TODO QGIS 4: Remove in favor of _QGIS_VERSION_INT
-#define VERSION_INT 33403
-#define _QGIS_VERSION_INT 33403
-#define ABISYM(x) x ## 33403
+#define VERSION_INT 33600
+#define _QGIS_VERSION_INT 33600
+#define ABISYM(x) x ## 33600
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Prizren"
+#define RELEASE_NAME "Maidenhead"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
@@ -31,7 +31,7 @@
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
 #define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.34.3"
+#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.36.0"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
@@ -46,10 +46,10 @@
 
 #define QT_PLUGINS_DIR "/usr/lib/qt/plugins"
 
-#define PYTHON_VERSION "3.11.6"
+#define PYTHON_VERSION "3.11.8"
 #define PYTHON_VERSION_MAJOR "3"
 #define PYTHON_VERSION_MINOR "11"
-#define PYTHON_VERSION_PATCH "6"
+#define PYTHON_VERSION_PATCH "8"
 
 #define PROJ_VERSION_MAJOR 9
 #define PROJ_VERSION_MINOR 3
@@ -98,6 +98,8 @@
 
 /* #undef HAVE_QTGAMEPAD */
 
+#define HAVE_QTPRINTER
+
 /* #undef HAVE_STATIC_PROVIDERS */
 
 #define HAVE_EPT
@@ -105,12 +107,12 @@
 #define HAVE_COPC
 
 #define HAVE_PDAL_QGIS
-#define PDAL_VERSION "2.6.2"
+#define PDAL_VERSION "2.6.3"
 #define PDAL_VERSION_MAJOR_INT 2
 #define PDAL_VERSION_MINOR_INT 6
 #define PDAL_VERSION_MAJOR "2"
 #define PDAL_VERSION_MINOR "6"
-#define PDAL_VERSION_MICRO "2"
+#define PDAL_VERSION_MICRO "3"
 
 #define HAVE_DRACO
 
@@ -120,7 +122,11 @@
 #define HAS_KDE_QT5_SMALL_CAPS_FIX
 #define HAS_KDE_QT5_FONT_STRETCH_FIX
 
+/* #undef HAVE_PDF4QT */
+
 #define HAVE_CRASH_HANDLER
 
+/* #undef HAVE_WEBENGINE */
+
 #endif
 
