@@ -46,7 +46,6 @@ namespace pal
  *
  * \note this class is not a part of public API yet. See notes in QgsLabelingEngine
  * \note not available in Python bindings
- * \since QGIS 2.12
  */
 class CORE_EXPORT QgsAbstractLabelProvider
 {
@@ -191,7 +190,7 @@ class CORE_EXPORT QgsAbstractLabelProvider
     //! Default priority of labels
     double mPriority = 0.5;
     //! Type of the obstacle of feature geometries
-    QgsLabelObstacleSettings::ObstacleType mObstacleType = QgsLabelObstacleSettings::PolygonBoundary;
+    QgsLabelObstacleSettings::ObstacleType mObstacleType = QgsLabelObstacleSettings::ObstacleType::PolygonBoundary;
     //! How to handle labels that would be upside down
     Qgis::UpsideDownLabelHandling mUpsidedownLabels = Qgis::UpsideDownLabelHandling::FlipUpsideDownLabels;
 
@@ -337,7 +336,6 @@ class CORE_EXPORT QgsLabelingEngineFeedback : public QgsFeedback
  * into feature loop vs providers with independent feature loop), split labeling
  * computation from drawing of labels, improved results class with label iterator).
  * \note not available in Python bindings
- * \since QGIS 2.12
  */
 class CORE_EXPORT QgsLabelingEngine
 {
@@ -362,7 +360,6 @@ class CORE_EXPORT QgsLabelingEngine
 
     /**
      * Returns a list of layers with providers in the engine.
-     * \since QGIS 3.0
      */
     QList< QgsMapLayer * > participatingLayers() const;
 
@@ -523,7 +520,6 @@ class CORE_EXPORT QgsStagedRenderLabelingEngine : public QgsLabelingEngine
  * \brief Contains helper utilities for working with QGIS' labeling engine.
  * \note this class is not a part of public API yet. See notes in QgsLabelingEngine
  * \note not available in Python bindings
- * \since QGIS 2.14
  */
 class CORE_EXPORT QgsLabelingUtils
 {
