@@ -35,7 +35,6 @@ class QgsGroupLayer;
  * While a layer tree group is typically used for hierarchical organisation of a QgsProject,
  * they can optionally be associated with a QgsGroupLayer for map rendering purposes.
  *
- * \since QGIS 2.4
  */
 class CORE_EXPORT QgsLayerTreeGroup : public QgsLayerTreeNode
 {
@@ -131,7 +130,6 @@ class CORE_EXPORT QgsLayerTreeGroup : public QgsLayerTreeNode
 
     /**
      * Find layer node representing the map layer. Searches recursively the whole sub-tree.
-     * \since QGIS 3.0
      */
     QgsLayerTreeLayer *findLayer( QgsMapLayer *layer ) const;
 
@@ -145,6 +143,19 @@ class CORE_EXPORT QgsLayerTreeGroup : public QgsLayerTreeNode
      */
     QList<QgsLayerTreeLayer *> findLayers() const;
 
+    /**
+     * Reorders layers in the group to match the order specified by \a order.
+     *
+     * Only layers which are direct children of this group will be reordered, other
+     * layers will be ignored.
+     *
+     * \note This method does not recursively reorder child groups.
+     * \note Matching layers will be moved to the start of the group, with any existing
+     * non-matching layers and group nodes moved to sit after the re-ordered matching layers.
+     *
+     * \since QGIS 3.30
+     */
+    void reorderGroupLayers( const QList< QgsMapLayer * > &order );
 
     /**
      * Returns an ordered list of map layers in the group, ignoring any layers which
@@ -179,7 +190,6 @@ class CORE_EXPORT QgsLayerTreeGroup : public QgsLayerTreeNode
     /**
      * Read group (tree) from XML element <layer-tree-group> and return the newly created group (or NULLPTR on error).
      * Also resolves textual references to layers from the project (calls resolveReferences() internally).
-     * \since QGIS 3.0
      */
     static QgsLayerTreeGroup *readXml( QDomElement &element, const QgsProject *project, const QgsReadWriteContext &context ) SIP_FACTORY;
 
@@ -206,7 +216,6 @@ class CORE_EXPORT QgsLayerTreeGroup : public QgsLayerTreeNode
 
     /**
      * Calls resolveReferences() on child tree nodes
-     * \since QGIS 3.0
      */
     void resolveReferences( const QgsProject *project, bool looseMatching = false ) override;
 
@@ -217,7 +226,6 @@ class CORE_EXPORT QgsLayerTreeGroup : public QgsLayerTreeNode
 
     /**
      * Returns whether the group is mutually exclusive (only one child can be checked at a time)
-     * \since QGIS 2.12
      */
     bool isMutuallyExclusive() const;
 
@@ -225,7 +233,6 @@ class CORE_EXPORT QgsLayerTreeGroup : public QgsLayerTreeNode
      * Set whether the group is mutually exclusive (only one child can be checked at a time).
      * The initial child index determines which child should be initially checked. The default value
      * of -1 will determine automatically (either first one currently checked or none)
-     * \since QGIS 2.12
      */
     void setIsMutuallyExclusive( bool enabled, int initialChildIndex = -1 );
 
