@@ -23,7 +23,6 @@
 #include "qgsprocessingparameters.h"
 #include "qgsprocessingoutputs.h"
 #include "qgsprocessingcontext.h"
-#include "qgsfeaturesource.h"
 #include "qgsprocessingutils.h"
 #include <QString>
 #include <QVariant>
@@ -35,6 +34,7 @@ class QgsFeatureSink;
 class QgsProcessingModelAlgorithm;
 class QgsProcessingAlgorithmConfigurationWidget;
 class QgsMeshLayer;
+class QgsPointCloudLayer;
 
 #ifdef SIP_RUN
 % ModuleHeaderCode
@@ -46,7 +46,6 @@ class QgsMeshLayer;
  * \class QgsProcessingAlgorithm
  * \ingroup core
  * \brief Abstract base class for processing algorithms.
-  * \since QGIS 3.0
  */
 class CORE_EXPORT QgsProcessingAlgorithm
 {
@@ -64,27 +63,6 @@ class CORE_EXPORT QgsProcessingAlgorithm
 
   public:
 
-    //! Flags indicating how and when an algorithm operates and should be exposed to users
-    enum Flag
-    {
-      FlagHideFromToolbox = 1 << 1, //!< Algorithm should be hidden from the toolbox
-      FlagHideFromModeler = 1 << 2, //!< Algorithm should be hidden from the modeler
-      FlagSupportsBatch = 1 << 3,  //!< Algorithm supports batch mode
-      FlagCanCancel = 1 << 4, //!< Algorithm can be canceled
-      FlagRequiresMatchingCrs = 1 << 5, //!< Algorithm requires that all input layers have matching coordinate reference systems
-      FlagNoThreading = 1 << 6, //!< Algorithm is not thread safe and cannot be run in a background thread, e.g. for algorithms which manipulate the current project, layer selections, or with external dependencies which are not thread-safe.
-      FlagDisplayNameIsLiteral = 1 << 7, //!< Algorithm's display name is a static literal string, and should not be translated or automatically formatted. For use with algorithms named after commands, e.g. GRASS 'v.in.ogr'.
-      FlagSupportsInPlaceEdits = 1 << 8, //!< Algorithm supports in-place editing
-      FlagKnownIssues = 1 << 9, //!< Algorithm has known issues
-      FlagCustomException = 1 << 10, //!< Algorithm raises custom exception notices, don't use the standard ones
-      FlagPruneModelBranchesBasedOnAlgorithmResults = 1 << 11, //!< Algorithm results will cause remaining model branches to be pruned based on the results of running the algorithm
-      FlagSkipGenericModelLogging = 1 << 12, //!< When running as part of a model, the generic algorithm setup and results logging should be skipped
-      FlagNotAvailableInStandaloneTool = 1 << 13, //!< Algorithm should not be available from the standalone "qgis_process" tool. Used to flag algorithms which make no sense outside of the QGIS application, such as "select by..." style algorithms.
-      FlagRequiresProject = 1 << 14, //!< The algorithm requires that a valid QgsProject is available from the processing context in order to execute
-      FlagDeprecated = FlagHideFromToolbox | FlagHideFromModeler, //!< Algorithm is deprecated
-    };
-    Q_DECLARE_FLAGS( Flags, Flag )
-
     /**
      * Constructor for QgsProcessingAlgorithm.
      *
@@ -144,7 +122,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \see group()
      * \see tags()
     */
-    virtual QString name() const = 0;
+    virtual QString name() const = 0 SIP_HOLDGIL;
 
     /**
      * Returns the unique ID for the algorithm, which is a combination of the algorithm
@@ -152,7 +130,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \see name()
      * \see provider()
      */
-    QString id() const;
+    QString id() const SIP_HOLDGIL;
 
     /**
      * Returns the translated algorithm name, which should be used for any user-visible display
@@ -164,20 +142,20 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \see name()
      * \see shortDescription()
      */
-    virtual QString displayName() const = 0;
+    virtual QString displayName() const = 0 SIP_HOLDGIL;
 
     /**
      * Returns an optional translated short description of the algorithm. This should be
      * at most a single sentence, e.g. "Converts 2D features to 3D by sampling a DEM raster."
      * \since QGIS 3.2
      */
-    virtual QString shortDescription() const;
+    virtual QString shortDescription() const SIP_HOLDGIL;
 
     /**
      * Returns a list of tags which relate to the algorithm, and are used to assist users in searching
      * for suitable algorithms. These tags should be localised.
     */
-    virtual QStringList tags() const { return QStringList(); }
+    virtual QStringList tags() const SIP_HOLDGIL { return QStringList(); }
 
     /**
      * Returns a localised short helper string for the algorithm. This string should provide a basic description
@@ -185,7 +163,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \see helpString()
      * \see helpUrl()
      */
-    virtual QString shortHelpString() const;
+    virtual QString shortHelpString() const SIP_HOLDGIL;
 
     /**
      * Returns a localised help string for the algorithm. Algorithm subclasses should implement either
@@ -194,26 +172,26 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \see shortHelpString()
      * \deprecated Unused, will be removed in QGIS 4.0
      */
-    Q_DECL_DEPRECATED virtual QString helpString() const SIP_DEPRECATED;
+    Q_DECL_DEPRECATED virtual QString helpString() const SIP_HOLDGIL SIP_DEPRECATED;
 
     /**
      * Returns a url pointing to the algorithm's help page.
      * \see helpString()
      * \see shortHelpString()
      */
-    virtual QString helpUrl() const;
+    virtual QString helpUrl() const SIP_HOLDGIL;
 
     /**
      * Returns an icon for the algorithm.
      * \see svgIconPath()
     */
-    virtual QIcon icon() const;
+    virtual QIcon icon() const SIP_HOLDGIL;
 
     /**
      * Returns a path to an SVG version of the algorithm's icon.
      * \see icon()
      */
-    virtual QString svgIconPath() const;
+    virtual QString svgIconPath() const SIP_HOLDGIL;
 
     /**
      * Returns the name of the group this algorithm belongs to. This string
@@ -221,7 +199,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \see groupId()
      * \see tags()
     */
-    virtual QString group() const { return QString(); }
+    virtual QString group() const SIP_HOLDGIL { return QString(); }
 
     /**
      * Returns the unique ID of the group this algorithm belongs to. This string
@@ -230,13 +208,13 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * alphanumeric characters only and no spaces or other formatting characters.
      * \see group()
      */
-    virtual QString groupId() const { return QString(); }
+    virtual QString groupId() const SIP_HOLDGIL { return QString(); }
 
     /**
      * Returns the flags indicating how and when the algorithm operates and should be exposed to users.
      * Default flags are FlagSupportsBatch and FlagCanCancel.
      */
-    virtual Flags flags() const;
+    virtual Qgis::ProcessingAlgorithmFlags flags() const SIP_HOLDGIL;
 
     /**
      * Returns TRUE if the algorithm can execute. Algorithm subclasses can return FALSE
@@ -268,7 +246,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
     /**
      * Returns the provider to which this algorithm belongs.
      */
-    QgsProcessingProvider *provider() const;
+    QgsProcessingProvider *provider() const SIP_HOLDGIL;
 
     /**
      * Returns an ordered list of parameter definitions utilized by the algorithm.
@@ -276,56 +254,47 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \see parameterDefinition()
      * \see destinationParameterDefinitions()
      */
-    QgsProcessingParameterDefinitions parameterDefinitions() const { return mParameters; }
+    QgsProcessingParameterDefinitions parameterDefinitions() const SIP_HOLDGIL { return mParameters; }
 
     /**
      * Returns a matching parameter by \a name. Matching is done in a case-insensitive
      * manner, but exact case matches will be preferred.
      * \see parameterDefinitions()
      */
-    const QgsProcessingParameterDefinition *parameterDefinition( const QString &name ) const;
+    const QgsProcessingParameterDefinition *parameterDefinition( const QString &name ) const SIP_HOLDGIL;
 
     /**
      * Returns the number of visible (non-hidden) parameters defined by this
      * algorithm.
      */
-    int countVisibleParameters() const;
+    int countVisibleParameters() const SIP_HOLDGIL;
 
     /**
      * Returns a list of destination parameters definitions utilized by the algorithm.
      * \see QgsProcessingParameterDefinition::isDestination()
      * \see parameterDefinitions()
      */
-    QgsProcessingParameterDefinitions destinationParameterDefinitions() const;
+    QgsProcessingParameterDefinitions destinationParameterDefinitions() const SIP_HOLDGIL;
 
     /**
      * Returns an ordered list of output definitions utilized by the algorithm.
      * \see addOutput()
      * \see outputDefinition()
      */
-    QgsProcessingOutputDefinitions outputDefinitions() const { return mOutputs; }
+    QgsProcessingOutputDefinitions outputDefinitions() const SIP_HOLDGIL { return mOutputs; }
 
     /**
      * Returns a matching output by \a name. Matching is done in a case-insensitive
      * manner.
      * \see outputDefinitions()
      */
-    const QgsProcessingOutputDefinition *outputDefinition( const QString &name ) const;
+    const QgsProcessingOutputDefinition *outputDefinition( const QString &name ) const SIP_HOLDGIL;
 
     /**
      * Returns TRUE if this algorithm generates HTML outputs.
      */
-    bool hasHtmlOutputs() const;
+    bool hasHtmlOutputs() const SIP_HOLDGIL;
 
-    /**
-     * Property availability, used for QgsProcessingAlgorithm::VectorProperties
-     * in order to determine if properties are available or not
-     */
-    enum PropertyAvailability
-    {
-      NotAvailable, //!< Properties are not available
-      Available, //!< Properties are available
-    };
 
     /**
      * Properties of a vector source or sink used in an algorithm.
@@ -338,13 +307,13 @@ class CORE_EXPORT QgsProcessingAlgorithm
       QgsFields fields;
 
       //! Geometry (WKB) type
-      QgsWkbTypes::Type wkbType = QgsWkbTypes::Unknown;
+      Qgis::WkbType wkbType = Qgis::WkbType::Unknown;
 
       //! Coordinate Reference System
       QgsCoordinateReferenceSystem crs;
 
       //! Availability of the properties. By default properties are not available.
-      QgsProcessingAlgorithm::PropertyAvailability availability = QgsProcessingAlgorithm::NotAvailable;
+      Qgis::ProcessingPropertyAvailability availability = Qgis::ProcessingPropertyAvailability::NotAvailable;
     };
 
     /**
@@ -460,10 +429,33 @@ class CORE_EXPORT QgsProcessingAlgorithm
      */
     virtual QString asPythonCommand( const QVariantMap &parameters, QgsProcessingContext &context ) const;
 
+    /**
+     * Returns a command string which will execute the algorithm using the specified \a parameters
+     * via the command line qgis_process tool.
+     *
+     * Note that some combinations of parameter types and values cannot be represented as a qgis_process string.
+     *
+     * \param parameters algorithm parameters
+     * \param context processing context
+     * \param ok will be set to TRUE if the command was successfully generated
+     *
+     * \returns equivalent qgis_process command
+     *
+     * \since QGIS 3.24
+     */
+    virtual QString asQgisProcessCommand( const QVariantMap &parameters, QgsProcessingContext &context, bool &ok SIP_OUT ) const;
+
+    /**
+     * Returns a JSON serializable variant map containing the specified \a parameters and \a context settings.
+     *
+     * \since QGIS 3.24
+     */
+    virtual QVariantMap asMap( const QVariantMap &parameters, QgsProcessingContext &context ) const;
+
     /**
      * Associates this algorithm with its provider. No transfer of ownership is involved.
      */
-    void setProvider( QgsProcessingProvider *provider );
+    void setProvider( QgsProcessingProvider *provider ) SIP_HOLDGIL;
 
   protected:
 
@@ -511,13 +503,13 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \see initAlgorithm()
      * \see addOutput()
      */
-    bool addParameter( QgsProcessingParameterDefinition *parameterDefinition SIP_TRANSFER, bool createOutput = true );
+    bool addParameter( QgsProcessingParameterDefinition *parameterDefinition SIP_TRANSFER, bool createOutput = true ) SIP_HOLDGIL;
 
     /**
      * Removes the parameter with matching \a name from the algorithm, and deletes any existing
      * definition.
      */
-    void removeParameter( const QString &name );
+    void removeParameter( const QString &name ) SIP_HOLDGIL;
 
     /**
      * Adds an output \a definition to the algorithm. Ownership of the definition is transferred to the algorithm.
@@ -532,7 +524,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \see addParameter()
      * \see initAlgorithm()
      */
-    bool addOutput( QgsProcessingOutputDefinition *outputDefinition SIP_TRANSFER );
+    bool addOutput( QgsProcessingOutputDefinition *outputDefinition SIP_TRANSFER ) SIP_HOLDGIL;
 
     /**
      * Prepares the algorithm to run using the specified \a parameters. Algorithms should implement
@@ -706,7 +698,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \throws QgsProcessingException
      */
     QgsFeatureSink *parameterAsSink( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context, QString &destinationIdentifier SIP_OUT,
-                                     const QgsFields &fields, QgsWkbTypes::Type geometryType = QgsWkbTypes::NoGeometry, const QgsCoordinateReferenceSystem &crs = QgsCoordinateReferenceSystem(), QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags(), const QVariantMap &createOptions = QVariantMap(), const QStringList &datasourceOptions = QStringList(), const QStringList &layerOptions = QStringList() ) const SIP_THROW( QgsProcessingException ) SIP_FACTORY;
+                                     const QgsFields &fields, Qgis::WkbType geometryType = Qgis::WkbType::NoGeometry, const QgsCoordinateReferenceSystem &crs = QgsCoordinateReferenceSystem(), QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags(), const QVariantMap &createOptions = QVariantMap(), const QStringList &datasourceOptions = QStringList(), const QStringList &layerOptions = QStringList() ) const SIP_THROW( QgsProcessingException ) SIP_FACTORY;
 
     /**
      * Evaluates the parameter with matching \a name to a feature source.
@@ -734,7 +726,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * to use parameterAsCompatibleSourceLayerPathAndLayerName() which may avoid conversion in more situations.
      */
     QString parameterAsCompatibleSourceLayerPath( const QVariantMap &parameters, const QString &name,
-        QgsProcessingContext &context, const QStringList &compatibleFormats, const QString &preferredFormat = QString( "shp" ), QgsProcessingFeedback *feedback = nullptr );
+        QgsProcessingContext &context, const QStringList &compatibleFormats, const QString &preferredFormat = QString( "shp" ), QgsProcessingFeedback *feedback = nullptr ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a source vector layer file path and layer name of compatible format.
@@ -766,7 +758,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \since QGIS 3.10
      */
     QString parameterAsCompatibleSourceLayerPathAndLayerName( const QVariantMap &parameters, const QString &name,
-        QgsProcessingContext &context, const QStringList &compatibleFormats, const QString &preferredFormat = QString( "shp" ), QgsProcessingFeedback *feedback = nullptr, QString *layerName SIP_OUT = nullptr );
+        QgsProcessingContext &context, const QStringList &compatibleFormats, const QString &preferredFormat = QString( "shp" ), QgsProcessingFeedback *feedback = nullptr, QString *layerName SIP_OUT = nullptr ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a map layer.
@@ -797,7 +789,6 @@ class CORE_EXPORT QgsProcessingAlgorithm
      */
     QgsMeshLayer *parameterAsMeshLayer( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
-
     /**
      * Evaluates the parameter with matching \a name to a output layer destination.
      */
@@ -844,14 +835,14 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * \see parameterAsExtent()
      */
     QgsGeometry parameterAsExtentGeometry( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context,
-                                           const QgsCoordinateReferenceSystem &crs = QgsCoordinateReferenceSystem() );
+                                           const QgsCoordinateReferenceSystem &crs = QgsCoordinateReferenceSystem() ) const;
 
     /**
      * Returns the coordinate reference system associated with an extent parameter value.
      *
      * \see parameterAsExtent()
      */
-    QgsCoordinateReferenceSystem parameterAsExtentCrs( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context );
+    QgsCoordinateReferenceSystem parameterAsExtentCrs( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a point.
@@ -869,7 +860,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      *
      * \see parameterAsPoint()
      */
-    QgsCoordinateReferenceSystem parameterAsPointCrs( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context );
+    QgsCoordinateReferenceSystem parameterAsPointCrs( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a geometry.
@@ -887,9 +878,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      *
      * \see parameterAsGeometry()
      */
-    QgsCoordinateReferenceSystem parameterAsGeometryCrs( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context );
-
-
+    QgsCoordinateReferenceSystem parameterAsGeometryCrs( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a file/folder name.
@@ -904,8 +893,9 @@ class CORE_EXPORT QgsProcessingAlgorithm
 
     /**
      * Evaluates the parameter with matching \a name to a list of map layers.
+     * The \a flags are used to set options for loading layers (e.g. skip index generation).
      */
-    QList< QgsMapLayer *> parameterAsLayerList( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
+    QList< QgsMapLayer *> parameterAsLayerList( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context, QgsProcessing::LayerOptionsFlags flags = QgsProcessing::LayerOptionsFlags() ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a list of files (for QgsProcessingParameterMultipleLayers in QgsProcessing:TypeFile mode).
@@ -921,8 +911,17 @@ class CORE_EXPORT QgsProcessingAlgorithm
 
     /**
      * Evaluates the parameter with matching \a name to a list of fields.
+     *
+     * \deprecated use parameterAsStrings() instead.
      */
-    QStringList parameterAsFields( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
+    Q_DECL_DEPRECATED QStringList parameterAsFields( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const SIP_DEPRECATED;
+
+    /**
+     * Evaluates the parameter with matching \a name to a list of strings (e.g. field names or point cloud attributes).
+     *
+     * \since QGIS 3.32
+     */
+    QStringList parameterAsStrings( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a print layout.
@@ -949,36 +948,62 @@ class CORE_EXPORT QgsProcessingAlgorithm
      *
      * \since QGIS 3.10
      */
-    QColor parameterAsColor( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context );
+    QColor parameterAsColor( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a connection name string.
      *
      * \since QGIS 3.14
      */
-    QString parameterAsConnectionName( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context );
+    QString parameterAsConnectionName( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a database schema name string.
      *
      * \since QGIS 3.14
      */
-    QString parameterAsSchema( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context );
+    QString parameterAsSchema( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a database table name string.
      *
      * \since QGIS 3.14
      */
-    QString parameterAsDatabaseTableName( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context );
+    QString parameterAsDatabaseTableName( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
     /**
      * Evaluates the parameter with matching \a name to a DateTime, or returns an invalid date time if the parameter was not set.
      *
      * \since QGIS 3.14
      */
-    QDateTime parameterAsDateTime( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context );
+    QDateTime parameterAsDateTime( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
+    /**
+     * Evaluates the parameter with matching \a name to a point cloud layer.
+     * The \a flags are used to set options for loading layer (e.g. skip index generation).
+     *
+     * Layers will either be taken from \a context's active project, or loaded from external
+     * sources and stored temporarily in the \a context. In either case, callers do not
+     * need to handle deletion of the returned layer.
+     *
+     * \since QGIS 3.22
+     */
+    QgsPointCloudLayer *parameterAsPointCloudLayer( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context, QgsProcessing::LayerOptionsFlags flags = QgsProcessing::LayerOptionsFlags() ) const;
+
+    /**
+     * Evaluates the parameter with matching \a name to an annotation layer.
+     *
+     * Annotation layers will be taken from \a context's active project. Callers do not
+     * need to handle deletion of the returned layer.
+     *
+     * \warning Working with annotation layers is generally not thread safe (unless the layers are from
+     * a QgsProject loaded directly in a background thread). Ensure your algorithm returns the
+     * QgsProcessingAlgorithm::FlagNoThreading flag or only accesses annotation layers from a prepareAlgorithm()
+     * or postProcessAlgorithm() step.
+     *
+     * \since QGIS 3.22
+     */
+    QgsAnnotationLayer *parameterAsAnnotationLayer( const QVariantMap &parameters, const QString &name, QgsProcessingContext &context ) const;
 
     /**
      * Returns a user-friendly string to use as an error when a source parameter could
@@ -990,6 +1015,7 @@ class CORE_EXPORT QgsProcessingAlgorithm
      *
      * \see invalidRasterError()
      * \see invalidSinkError()
+     * \see invalidPointCloudError()
      * \since QGIS 3.2
      */
     static QString invalidSourceError( const QVariantMap &parameters, const QString &name );
@@ -999,11 +1025,12 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * not be loaded.
      *
      * The \a parameters argument should give the algorithms parameter map, and the \a name
-     * should correspond to the invalid source parameter name.
+     * should correspond to the invalid raster parameter name.
      *
      *
      * \see invalidSourceError()
      * \see invalidSinkError()
+     * \see invalidPointCloudError()
      * \since QGIS 3.2
      */
     static QString invalidRasterError( const QVariantMap &parameters, const QString &name );
@@ -1013,15 +1040,44 @@ class CORE_EXPORT QgsProcessingAlgorithm
      * not be created.
      *
      * The \a parameters argument should give the algorithms parameter map, and the \a name
-     * should correspond to the invalid source parameter name.
+     * should correspond to the invalid sink parameter name.
      *
      *
      * \see invalidSourceError()
      * \see invalidRasterError()
+     * \see invalidPointCloudError()
      * \since QGIS 3.2
      */
     static QString invalidSinkError( const QVariantMap &parameters, const QString &name );
 
+    /**
+     * Returns a user-friendly string to use as an error when a point cloud layer input could
+     * not be loaded.
+     *
+     * The \a parameters argument should give the algorithms parameter map, and the \a name
+     * should correspond to the invalid point cloud parameter name.
+     *
+     *
+     * \see invalidSourceError()
+     * \see invalidSinkError()
+     * \see invalidRasterError()
+     * \since QGIS 3.32
+     */
+    static QString invalidPointCloudError( const QVariantMap &parameters, const QString &name );
+
+    /**
+     * Returns a user-friendly string to use as an error when a feature cannot be
+     * written into a sink.
+     *
+     * The \a sink argument is the sink into which the feature cannot be written.
+     *
+     * The \a parameters argument should give the algorithms parameter map, and the \a name
+     * should correspond to the sink parameter name.
+     *
+     * \since QGIS 3.22
+     */
+    static QString writeFeatureError( QgsFeatureSink *sink, const QVariantMap &parameters, const QString &name );
+
     /**
      * Checks whether this algorithm supports in-place editing on the given \a layer
      * Default implementation returns FALSE.
@@ -1054,8 +1110,6 @@ class CORE_EXPORT QgsProcessingAlgorithm
 #endif
 
 };
-Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProcessingAlgorithm::Flags )
-
 
 
 /**
@@ -1078,7 +1132,6 @@ Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProcessingAlgorithm::Flags )
  * (for instance allowing automatic multi-thread processing of the algorithm, or use of the
  * algorithm in "chains", avoiding the need for temporary outputs in multi-step models).
  *
- * \since QGIS 3.0
  */
 
 class CORE_EXPORT QgsProcessingFeatureBasedAlgorithm : public QgsProcessingAlgorithm
@@ -1090,7 +1143,7 @@ class CORE_EXPORT QgsProcessingFeatureBasedAlgorithm : public QgsProcessingAlgor
       */
     QgsProcessingFeatureBasedAlgorithm() = default;
 
-    QgsProcessingAlgorithm::Flags flags() const override;
+    Qgis::ProcessingAlgorithmFlags flags() const override SIP_HOLDGIL;
 
     /**
      * Processes an individual input \a feature from the source. Algorithms should implement their
@@ -1128,7 +1181,7 @@ class CORE_EXPORT QgsProcessingFeatureBasedAlgorithm : public QgsProcessingAlgor
      *
      * \since QGIS 3.12
      */
-    virtual QString inputParameterName() const;
+    virtual QString inputParameterName() const SIP_HOLDGIL;
 
     /**
      * Returns the translated description of the parameter corresponding to the input layer.
@@ -1137,39 +1190,39 @@ class CORE_EXPORT QgsProcessingFeatureBasedAlgorithm : public QgsProcessingAlgor
      *
      * \since QGIS 3.12
      */
-    virtual QString inputParameterDescription() const;
+    virtual QString inputParameterDescription() const SIP_HOLDGIL;
 
     /**
      * Returns the translated, user visible name for any layers created by this algorithm.
      * This name will be used as the default name when loading the resultant layer into a
      * QGIS project.
      */
-    virtual QString outputName() const = 0;
+    virtual QString outputName() const = 0 SIP_HOLDGIL;
 
     /**
      * Returns the valid input layer types for the source layer for this algorithm.
      * By default vector layers with any geometry types (excluding non-spatial, geometryless layers)
      * are accepted.
      */
-    virtual QList<int> inputLayerTypes() const;
+    virtual QList<int> inputLayerTypes() const SIP_HOLDGIL;
 
     /**
      * Returns the layer type for layers generated by this algorithm, if
      * this is possible to determine in advance.
      */
-    virtual QgsProcessing::SourceType outputLayerType() const;
+    virtual Qgis::ProcessingSourceType outputLayerType() const SIP_HOLDGIL;
 
     /**
      * Returns the processing feature source flags to be used in the algorithm.
      */
-    virtual QgsProcessingFeatureSource::Flag sourceFlags() const;
+    virtual Qgis::ProcessingFeatureSourceFlags sourceFlags() const SIP_HOLDGIL;
 
     /**
      * Returns the feature sink flags to be used for the output.
      *
      * \since QGIS 3.4.1
      */
-    virtual QgsFeatureSink::SinkFlags sinkFlags() const;
+    virtual QgsFeatureSink::SinkFlags sinkFlags() const SIP_HOLDGIL;
 
     /**
      * Maps the input WKB geometry type (\a inputWkbType) to the corresponding
@@ -1178,7 +1231,7 @@ class CORE_EXPORT QgsProcessingFeatureBasedAlgorithm : public QgsProcessingAlgor
      * This is called once by the base class when creating the output sink for the algorithm (i.e. it is
      * not called once per feature processed).
      */
-    virtual QgsWkbTypes::Type outputWkbType( QgsWkbTypes::Type inputWkbType ) const;
+    virtual Qgis::WkbType outputWkbType( Qgis::WkbType inputWkbType ) const SIP_HOLDGIL;
 
     /**
      * Maps the input source fields (\a inputFields) to corresponding
@@ -1190,7 +1243,7 @@ class CORE_EXPORT QgsProcessingFeatureBasedAlgorithm : public QgsProcessingAlgor
      * This is called once by the base class when creating the output sink for the algorithm (i.e. it is
      * not called once per feature processed).
      */
-    virtual QgsFields outputFields( const QgsFields &inputFields ) const;
+    virtual QgsFields outputFields( const QgsFields &inputFields ) const SIP_HOLDGIL;
 
     /**
      * Maps the input source coordinate reference system (\a inputCrs) to a corresponding
@@ -1200,7 +1253,7 @@ class CORE_EXPORT QgsProcessingFeatureBasedAlgorithm : public QgsProcessingAlgor
      * This is called once by the base class when creating the output sink for the algorithm (i.e. it is
      * not called once per feature processed).
      */
-    virtual QgsCoordinateReferenceSystem outputCrs( const QgsCoordinateReferenceSystem &inputCrs ) const;
+    virtual QgsCoordinateReferenceSystem outputCrs( const QgsCoordinateReferenceSystem &inputCrs ) const SIP_HOLDGIL;
 
     /**
      * Initializes any extra parameters added by the algorithm subclass. There is no need
@@ -1213,7 +1266,7 @@ class CORE_EXPORT QgsProcessingFeatureBasedAlgorithm : public QgsProcessingAlgor
      * Returns the source's coordinate reference system. This will only return a valid CRS when
      * called from a subclasses' processFeature() implementation.
      */
-    QgsCoordinateReferenceSystem sourceCrs() const;
+    QgsCoordinateReferenceSystem sourceCrs() const SIP_HOLDGIL;
 
 
     QVariantMap processAlgorithm( const QVariantMap &parameters, QgsProcessingContext &context, QgsProcessingFeedback *feedback ) override SIP_THROW( QgsProcessingException );
@@ -1255,5 +1308,3 @@ class CORE_EXPORT QgsProcessingFeatureBasedAlgorithm : public QgsProcessingAlgor
 // clazy:excludeall=qstring-allocations
 
 #endif // QGSPROCESSINGALGORITHM_H
-
-
