@@ -67,7 +67,7 @@ class IdwInterpolation(QgisAlgorithm):
         self.addParameter(ParameterInterpolationData(self.INTERPOLATION_DATA,
                                                      self.tr('Input layer(s)')))
         self.addParameter(QgsProcessingParameterNumber(self.DISTANCE_COEFFICIENT,
-                                                       self.tr('Distance coefficient P'), type=QgsProcessingParameterNumber.Double,
+                                                       self.tr('Distance coefficient P'), type=QgsProcessingParameterNumber.Type.Double,
                                                        minValue=0.0, maxValue=99.99, defaultValue=2.0))
         self.addParameter(QgsProcessingParameterExtent(self.EXTENT,
                                                        self.tr('Extent'),
@@ -84,14 +84,14 @@ class IdwInterpolation(QgisAlgorithm):
                                                   self.tr('Number of columns'),
                                                   optional=True,
                                                   minValue=0, maxValue=10000000)
-        cols_param.setFlags(cols_param.flags() | QgsProcessingParameterDefinition.FlagHidden)
+        cols_param.setFlags(cols_param.flags() | QgsProcessingParameterDefinition.Flag.FlagHidden)
         self.addParameter(cols_param)
 
         rows_param = QgsProcessingParameterNumber(self.ROWS,
                                                   self.tr('Number of rows'),
                                                   optional=True,
                                                   minValue=0, maxValue=10000000)
-        rows_param.setFlags(rows_param.flags() | QgsProcessingParameterDefinition.FlagHidden)
+        rows_param.setFlags(rows_param.flags() | QgsProcessingParameterDefinition.Flag.FlagHidden)
         self.addParameter(rows_param)
 
         self.addParameter(QgsProcessingParameterRasterDestination(self.OUTPUT,
@@ -135,16 +135,16 @@ class IdwInterpolation(QgisAlgorithm):
 
             data.valueSource = int(v[1])
             data.interpolationAttribute = int(v[2])
-            if data.valueSource == QgsInterpolator.ValueAttribute and data.interpolationAttribute == -1:
+            if data.valueSource == QgsInterpolator.ValueSource.ValueAttribute and data.interpolationAttribute == -1:
                 raise QgsProcessingException(self.tr(
                     'Layer {} is set to use a value attribute, but no attribute was set').format(i + 1))
 
             if v[3] == '0':
-                data.sourceType = QgsInterpolator.SourcePoints
+                data.sourceType = QgsInterpolator.SourceType.SourcePoints
             elif v[3] == '1':
-                data.sourceType = QgsInterpolator.SourceStructureLines
+                data.sourceType = QgsInterpolator.SourceType.SourceStructureLines
             else:
-                data.sourceType = QgsInterpolator.SourceBreakLines
+                data.sourceType = QgsInterpolator.SourceType.SourceBreakLines
             layerData.append(data)
 
         interpolator = QgsIDWInterpolator(layerData)
